/* oled_clock
 *
 * Csoportos adatbeolvasás egy DS32231 RTC-ről.
 * A beolvasott adatok az RTC-ből kiolvasott
 * aktuális időt és dátumot adják meg.
 * Az alábbi programban az időt egy SSD1306
 * 128x32 felbontású OLED kijelzőn jelenítjük meg.
 * Az OLED kijelző is az I2C buszra csatlakozik
 *
 * Az I2C kapcsolathoz az I2C1 csatornát és a 
 * PB8 (SCL), PB9 (SDA) kivezetéseket használjuk.
 * A kommunikáció standard módban (100 kHz), 1:1
 * kitöltéssel (F/S = 0, DUTY = 0) megy.
 *
 * Kapcsolás:
 *   --------        -------------        ------------------------
 *  | STM32  |      | DS3231 RTC  |      | SSD1306 128x32         |
 *  |    PB9 |------| SDA         |------| SDA  2020-FEB-13 THU   |
 *  |    PB8 |------| SCL         |------| SCL      _     _   _   |  
 *  |     5V |------| VCC         |------| VCC   |   | * | | | |  |
 *  |    GND |------| GND         |------| GND   |   | * |_| |_|  |
 *   --------        -------------        ------------------------
 * Felhasznált forrás: 
 * Mazidi et al., STM32 Arm Programming for Embedded Systems    
 * http://www.microdigitaled.com/ARM/STM_ARM_books.html
 *
 * Tested with Keil MDK-Lite v5.28 and STM32F1 DFP v2.3.0
 */
 
#include "stm32f10x.h"
#include "ssd1306.h"

#define SLAVE_ADDR 0x68             /* 1101 000.    DS3231 */
void displayTime(uint8_t *data);

#define dec1(x)     (x &0x0f)+48
#define dec10(x)    ((x>>4)&0x07)+48 

int main(void) {
  uint8_t timeDateReadback[15];

  uint8_t lasttime =0;
  SSD1306_Init();

  while (1) {
    I2C1_ReadMulti(SLAVE_ADDR<<1, 0, timeDateReadback, 7);
    if (timeDateReadback[0] != lasttime) {
      lasttime = timeDateReadback[0];
      displayTime(timeDateReadback);
    }
  }
}

void displayTime(uint8_t *timeDate) {
  char daysOfTheWeek[8][3] = {"   ", "MON", "TUE", "WED", "THU", "FRI", "SAT", "SUN"};
  char sDate[15], sTime[10];
  //--- A dátum kiíratása YYYY-MM-DD DOW formátumban ---
  sDate[0] = '2';	                               // Évszázad = 20xx
  sDate[1] = '0';	
  sDate[2] = dec10(timeDate[6]);                 // Évtizedek	
  sDate[3] = dec1(timeDate[6]);                  // Évek	
  sDate[4] = '-';	
  sDate[5] = dec10(timeDate[5]);                 // Tízhónapok	
  sDate[6] = dec1(timeDate[5]);                  // Hónapok	
  sDate[7] = '-';
  sDate[8] = dec10(timeDate[4]);                 // Tíznapok
  sDate[9] = dec1(timeDate[4]);                  // Napok
  sDate[10] = ' ';
  for(int i=0; i<3; i++) {
    sDate[11+i] = daysOfTheWeek[timeDate[3]][i];
  }
  sDate[14] = 0;
  //--- Az idő kiíratása HH:MM:SS formátumban ---
  sTime[0] = dec10(timeDate[2]);                 // Tízórák
  sTime[1] = dec1(timeDate[2]);                  // Órák
  sTime[2] = ':';
  sTime[3] = dec10(timeDate[1]);                 // Tízpercek
  sTime[4] = dec1(timeDate[1]);                  // Percek
  sTime[5] = ':';
  sTime[6] = dec10(timeDate[0]);                 // Tízmásodpercek
  sTime[7] = dec1(timeDate[0]);	                 // másodpercek
  sTime[8] = 0;	
  SSD1306_GotoXY(5, 1);
  SSD1306_Puts(sDate, &Font_7x10, SSD1306_COLOR_WHITE);
  SSD1306_GotoXY(12, 12);
  SSD1306_Puts(sTime, &Font_11x18, SSD1306_COLOR_WHITE);
  SSD1306_UpdateScreen();                        // Megjelenítés
}



